@@ -1,6 +1,6 @@
 /*
 **  GNU Pth - The GNU Portable Threads
-**  Copyright (c) 1999-2002 Ralf S. Engelschall <rse@engelschall.com>
+**  Copyright (c) 1999-2003 Ralf S. Engelschall <rse@engelschall.com>
 **
 **  This file is part of GNU Pth, a non-preemptive thread scheduling
 **  library which can be found at http://www.gnu.org/software/pth/.
@@ -37,6 +37,7 @@ enum {
 struct pth_attr_st {
     pth_t        a_tid;
     int          a_prio;
+    int          a_dispatches;
     char         a_name[PTH_TCB_NAMELEN];
     int          a_joinable;
     unsigned int a_cancelstate;
@@ -51,9 +52,9 @@ pth_attr_t pth_attr_of(pth_t t)
     pth_attr_t a;
 
     if (t == NULL)
-        return_errno(FALSE, EINVAL);
+        return pth_error((pth_attr_t)NULL, EINVAL);
     if ((a = (pth_attr_t)malloc(sizeof(struct pth_attr_st))) == NULL)
-        return_errno(NULL, ENOMEM);
+        return pth_error((pth_attr_t)NULL, ENOMEM);
     a->a_tid = t;
     return a;
 }
@@ -63,7 +64,7 @@ pth_attr_t pth_attr_new(void)
     pth_attr_t a;
 
     if ((a = (pth_attr_t)malloc(sizeof(struct pth_attr_st))) == NULL)
-        return_errno(NULL, ENOMEM);
+        return pth_error((pth_attr_t)NULL, ENOMEM);
     a->a_tid = NULL;
     pth_attr_init(a);
     return a;
@@ -72,7 +73,7 @@ pth_attr_t pth_attr_new(void)
 int pth_attr_destroy(pth_attr_t a)
 {
     if (a == NULL)
-        return_errno(FALSE, EINVAL);
+        return pth_error(FALSE, EINVAL);
     free(a);
     return TRUE;
 }
@@ -80,11 +81,12 @@ int pth_attr_destroy(pth_attr_t a)
 int pth_attr_init(pth_attr_t a)
 {
     if (a == NULL)
-        return_errno(FALSE, EINVAL);
+        return pth_error(FALSE, EINVAL);
     if (a->a_tid != NULL)
-        return_errno(FALSE, EPERM);
+        return pth_error(FALSE, EPERM);
     a->a_prio = PTH_PRIO_STD;
     pth_util_cpystrn(a->a_name, "unknown", PTH_TCB_NAMELEN);
+    a->a_dispatches = 0;
     a->a_joinable = TRUE;
     a->a_cancelstate = PTH_CANCEL_DEFAULT;
     a->a_stacksize = 64*1024;
@@ -117,7 +119,7 @@ int pth_attr_set(pth_attr_t a, int op, ...)
 intern int pth_attr_ctrl(int cmd, pth_attr_t a, int op, va_list ap)
 {
     if (a == NULL)
-        return_errno(FALSE, EINVAL);
+        return pth_error(FALSE, EINVAL);
     switch (op) {
         case PTH_ATTR_PRIO: {
             /* priority */
@@ -149,6 +151,19 @@ intern int pth_attr_ctrl(int cmd, pth_attr_t a, int op, va_list ap)
             }
             break;
         }
+        case PTH_ATTR_DISPATCHES: {
+            /* incremented on every context switch */
+            int val, *src, *dst;
+            if (cmd == PTH_ATTR_SET) {
+                src = &val; val = va_arg(ap, int);
+                dst = (a->a_tid != NULL ? &a->a_tid->dispatches : &a->a_dispatches);
+            }
+            else {
+                src = (a->a_tid != NULL ? &a->a_tid->dispatches : &a->a_dispatches);
+                dst = va_arg(ap, int *);
+            }
+            *dst = *src;
+        }
         case PTH_ATTR_JOINABLE: {
             /* detachment type */
             int val, *src, *dst;
@@ -182,7 +197,7 @@ intern int pth_attr_ctrl(int cmd, pth_attr_t a, int op, va_list ap)
             unsigned int val, *src, *dst;
             if (cmd == PTH_ATTR_SET) {
                 if (a->a_tid != NULL)
-                    return_errno(FALSE, EPERM);
+                    return pth_error(FALSE, EPERM);
                 src = &val; val = va_arg(ap, unsigned int);
                 dst = &a->a_stacksize;
             }
@@ -198,7 +213,7 @@ intern int pth_attr_ctrl(int cmd, pth_attr_t a, int op, va_list ap)
             char *val, **src, **dst;
             if (cmd == PTH_ATTR_SET) {
                 if (a->a_tid != NULL)
-                    return_errno(FALSE, EPERM);
+                    return pth_error(FALSE, EPERM);
                 src = &val; val = va_arg(ap, char *);
                 dst = &a->a_stackaddr;
             }
@@ -212,7 +227,7 @@ intern int pth_attr_ctrl(int cmd, pth_attr_t a, int op, va_list ap)
         case PTH_ATTR_TIME_SPAWN: {
             pth_time_t *dst;
             if (cmd == PTH_ATTR_SET)
-                return_errno(FALSE, EPERM);
+                return pth_error(FALSE, EPERM);
             dst = va_arg(ap, pth_time_t *);
             if (a->a_tid != NULL)
                 pth_time_set(dst, &a->a_tid->spawned);
@@ -223,7 +238,7 @@ intern int pth_attr_ctrl(int cmd, pth_attr_t a, int op, va_list ap)
         case PTH_ATTR_TIME_LAST: {
             pth_time_t *dst;
             if (cmd == PTH_ATTR_SET)
-                return_errno(FALSE, EPERM);
+                return pth_error(FALSE, EPERM);
             dst = va_arg(ap, pth_time_t *);
             if (a->a_tid != NULL)
                 pth_time_set(dst, &a->a_tid->lastran);
@@ -234,7 +249,7 @@ intern int pth_attr_ctrl(int cmd, pth_attr_t a, int op, va_list ap)
         case PTH_ATTR_TIME_RAN: {
             pth_time_t *dst;
             if (cmd == PTH_ATTR_SET)
-                return_errno(FALSE, EPERM);
+                return pth_error(FALSE, EPERM);
             dst = va_arg(ap, pth_time_t *);
             if (a->a_tid != NULL)
                 pth_time_set(dst, &a->a_tid->running);
@@ -245,9 +260,9 @@ intern int pth_attr_ctrl(int cmd, pth_attr_t a, int op, va_list ap)
         case PTH_ATTR_START_FUNC: {
             void *(**dst)(void *);
             if (cmd == PTH_ATTR_SET)
-                return_errno(FALSE, EPERM);
+                return pth_error(FALSE, EPERM);
             if (a->a_tid == NULL)
-                return_errno(FALSE, EACCES);
+                return pth_error(FALSE, EACCES);
             dst = (void *(**)(void *))va_arg(ap, void *);
             *dst = a->a_tid->start_func;
             break;
@@ -255,9 +270,9 @@ intern int pth_attr_ctrl(int cmd, pth_attr_t a, int op, va_list ap)
         case PTH_ATTR_START_ARG: {
             void **dst;
             if (cmd == PTH_ATTR_SET)
-                return_errno(FALSE, EPERM);
+                return pth_error(FALSE, EPERM);
             if (a->a_tid == NULL)
-                return_errno(FALSE, EACCES);
+                return pth_error(FALSE, EACCES);
             dst = va_arg(ap, void **);
             *dst = a->a_tid->start_arg;
             break;
@@ -265,9 +280,9 @@ intern int pth_attr_ctrl(int cmd, pth_attr_t a, int op, va_list ap)
         case PTH_ATTR_STATE: {
             pth_state_t *dst;
             if (cmd == PTH_ATTR_SET)
-                return_errno(FALSE, EPERM);
+                return pth_error(FALSE, EPERM);
             if (a->a_tid == NULL)
-                return_errno(FALSE, EACCES);
+                return pth_error(FALSE, EACCES);
             dst = va_arg(ap, pth_state_t *);
             *dst = a->a_tid->state;
             break;
@@ -275,9 +290,9 @@ intern int pth_attr_ctrl(int cmd, pth_attr_t a, int op, va_list ap)
         case PTH_ATTR_EVENTS: {
             pth_event_t *dst;
             if (cmd == PTH_ATTR_SET)
-                return_errno(FALSE, EPERM);
+                return pth_error(FALSE, EPERM);
             if (a->a_tid == NULL)
-                return_errno(FALSE, EACCES);
+                return pth_error(FALSE, EACCES);
             dst = va_arg(ap, pth_event_t *);
             *dst = a->a_tid->events;
             break;
@@ -285,13 +300,13 @@ intern int pth_attr_ctrl(int cmd, pth_attr_t a, int op, va_list ap)
         case PTH_ATTR_BOUND: {
             int *dst;
             if (cmd == PTH_ATTR_SET)
-                return_errno(FALSE, EPERM);
+                return pth_error(FALSE, EPERM);
             dst = va_arg(ap, int *);
             *dst = (a->a_tid != NULL ? TRUE : FALSE);
             break;
         }
         default:
-            return_errno(FALSE, EINVAL);
+            return pth_error(FALSE, EINVAL);
     }
     return TRUE;
 }
