@@ -1,6 +1,6 @@
 /*
 **  GNU Pth - The GNU Portable Threads
-**  Copyright (c) 1999-2002 Ralf S. Engelschall <rse@engelschall.com>
+**  Copyright (c) 1999-2003 Ralf S. Engelschall <rse@engelschall.com>
 **
 **  This file is part of GNU Pth, a non-preemptive thread scheduling
 **  library which can be found at http://www.gnu.org/software/pth/.
@@ -56,15 +56,15 @@ int pth_cancel(pth_t thread)
     pth_pqueue_t *q;
 
     if (thread == NULL)
-        return_errno(FALSE, EINVAL);
+        return pth_error(FALSE, EINVAL);
 
     /* the current thread cannot be cancelled */
     if (thread == pth_current)
-        return_errno(FALSE, EINVAL);
+        return pth_error(FALSE, EINVAL);
 
     /* the thread has to be at least still alive */
     if (thread->state == PTH_STATE_DEAD)
-        return_errno(FALSE, EPERM);
+        return pth_error(FALSE, EPERM);
 
     /* now mark the thread as cancelled */
     thread->cancelreq = TRUE;
@@ -81,9 +81,9 @@ int pth_cancel(pth_t thread)
             default:                q = NULL;
         }
         if (q == NULL)
-            return_errno(FALSE, ESRCH);
+            return pth_error(FALSE, ESRCH);
         if (!pth_pqueue_contains(q, thread))
-            return_errno(FALSE, ESRCH);
+            return pth_error(FALSE, ESRCH);
         pth_pqueue_delete(q, thread);
 
         /* execute cleanups */
@@ -108,11 +108,11 @@ int pth_cancel(pth_t thread)
 int pth_abort(pth_t thread)
 {
     if (thread == NULL)
-        return_errno(FALSE, EINVAL);
+        return pth_error(FALSE, EINVAL);
 
     /* the current thread cannot be aborted */
     if (thread == pth_current)
-        return_errno(FALSE, EINVAL);
+        return pth_error(FALSE, EINVAL);
 
     if (thread->state == PTH_STATE_DEAD && thread->joinable) {
         /* if thread is already terminated, just join it */
