@@ -1,6 +1,6 @@
 /*
 **  GNU Pth - The GNU Portable Threads
-**  Copyright (c) 1999-2002 Ralf S. Engelschall <rse@engelschall.com>
+**  Copyright (c) 1999-2003 Ralf S. Engelschall <rse@engelschall.com>
 **
 **  This file is part of GNU Pth, a non-preemptive thread scheduling
 **  library which can be found at http://www.gnu.org/software/pth/.
@@ -44,9 +44,9 @@ int pth_cleanup_push(void (*func)(void *), void *arg)
     pth_cleanup_t *cleanup;
 
     if (func == NULL)
-        return_errno(FALSE, EINVAL);
+        return pth_error(FALSE, EINVAL);
     if ((cleanup = (pth_cleanup_t *)malloc(sizeof(pth_cleanup_t))) == NULL)
-        return_errno(FALSE, ENOMEM);
+        return pth_error(FALSE, ENOMEM);
     cleanup->func = func;
     cleanup->arg  = arg;
     cleanup->next = pth_current->cleanups;
