@@ -1,6 +1,6 @@
 /*
 **  GNU Pth - The GNU Portable Threads
-**  Copyright (c) 1999-2002 Ralf S. Engelschall <rse@engelschall.com>
+**  Copyright (c) 1999-2003 Ralf S. Engelschall <rse@engelschall.com>
 **
 **  This file is part of GNU Pth, a non-preemptive thread scheduling
 **  library which can be found at http://www.gnu.org/software/pth/.
@@ -43,15 +43,15 @@ int pth_key_create(pth_key_t *key, void (*func)(void *))
             return TRUE;
         }
     }
-    return_errno(FALSE, EAGAIN);
+    return pth_error(FALSE, EAGAIN);
 }
 
 int pth_key_delete(pth_key_t key)
 {
     if (key >= PTH_KEY_MAX)
-        return_errno(FALSE, EINVAL);
+        return pth_error(FALSE, EINVAL);
     if (!pth_keytab[key].used)
-        return_errno(FALSE, EINVAL);
+        return pth_error(FALSE, EINVAL);
     pth_keytab[key].used = FALSE;
     return TRUE;
 }
@@ -59,13 +59,13 @@ int pth_key_delete(pth_key_t key)
 int pth_key_setdata(pth_key_t key, const void *value)
 {
     if (key >= PTH_KEY_MAX)
-        return_errno(FALSE, EINVAL);
+        return pth_error(FALSE, EINVAL);
     if (!pth_keytab[key].used)
-        return_errno(FALSE, EINVAL);
+        return pth_error(FALSE, EINVAL);
     if (pth_current->data_value == NULL) {
         pth_current->data_value = (const void **)calloc(1, sizeof(void *)*PTH_KEY_MAX);
         if (pth_current->data_value == NULL)
-            return_errno(FALSE, ENOMEM);
+            return pth_error(FALSE, ENOMEM);
     }
     if (pth_current->data_value[key] == NULL) {
         if (value != NULL)
@@ -82,11 +82,11 @@ int pth_key_setdata(pth_key_t key, const void *value)
 void *pth_key_getdata(pth_key_t key)
 {
     if (key >= PTH_KEY_MAX)
-        return_errno(NULL, EINVAL);
+        return pth_error((void *)NULL, EINVAL);
     if (!pth_keytab[key].used)
-        return_errno(NULL, EINVAL);
+        return pth_error((void *)NULL, EINVAL);
     if (pth_current->data_value == NULL)
-        return NULL;
+        return (void *)NULL;
     return (void *)pth_current->data_value[key];
 }
 
