@@ -1,6 +1,6 @@
 /*
 **  GNU Pth - The GNU Portable Threads
-**  Copyright (c) 1999-2002 Ralf S. Engelschall <rse@engelschall.com>
+**  Copyright (c) 1999-2003 Ralf S. Engelschall <rse@engelschall.com>
 **
 **  This file is part of GNU Pth, a non-preemptive thread scheduling
 **  library which can be found at http://www.gnu.org/software/pth/.
@@ -34,7 +34,7 @@
 struct pth_event_st {
     struct pth_event_st *ev_next;
     struct pth_event_st *ev_prev;
-    int ev_occurred;
+    pth_status_t ev_status;
     int ev_type;
     int ev_goal;
     union {
@@ -92,7 +92,7 @@ pth_event_t pth_event(unsigned long spec, ...)
         ev = (pth_event_t)malloc(sizeof(struct pth_event_st));
     }
     if (ev == NULL)
-        return NULL; /* errno is already set */
+        return pth_error((pth_event_t)NULL, errno);
 
     /* create new event ring out of event or insert into existing ring */
     if (spec & PTH_MODE_CHAIN) {
@@ -108,14 +108,14 @@ pth_event_t pth_event(unsigned long spec, ...)
     }
 
     /* initialize common ingredients */
-    ev->ev_occurred = FALSE;
+    ev->ev_status = PTH_STATUS_PENDING;
 
     /* initialize event specific ingredients */
     if (spec & PTH_EVENT_FD) {
         /* filedescriptor event */
         int fd = va_arg(ap, int);
         if (!pth_util_fd_valid(fd))
-            return_errno(NULL, EBADF);
+            return pth_error((pth_event_t)NULL, EBADF);
         ev->ev_type = PTH_EVENT_FD;
         ev->ev_goal = (int)(spec & (PTH_UNTIL_FD_READABLE|\
                                     PTH_UNTIL_FD_WRITEABLE|\
@@ -201,7 +201,7 @@ pth_event_t pth_event(unsigned long spec, ...)
         ev->ev_args.FUNC.tv    = va_arg(ap, pth_time_t);
     }
     else
-        return_errno(NULL, EINVAL);
+        return pth_error((pth_event_t)NULL, EINVAL);
 
     va_end(ap);
 
@@ -213,7 +213,7 @@ pth_event_t pth_event(unsigned long spec, ...)
 unsigned long pth_event_typeof(pth_event_t ev)
 {
     if (ev == NULL)
-        return_errno(0, EINVAL);
+        return pth_error(0, EINVAL);
     return (ev->ev_type | ev->ev_goal);
 }
 
@@ -223,7 +223,7 @@ int pth_event_extract(pth_event_t ev, ...)
     va_list ap;
 
     if (ev == NULL)
-        return_errno(FALSE, EINVAL);
+        return pth_error(FALSE, EINVAL);
     va_start(ap, ev);
 
     /* extract event specific ingredients */
@@ -274,7 +274,7 @@ int pth_event_extract(pth_event_t ev, ...)
         *tv   = ev->ev_args.FUNC.tv;
     }
     else
-        return_errno(FALSE, EINVAL);
+        return pth_error(FALSE, EINVAL);
     va_end(ap);
     return TRUE;
 }
@@ -289,7 +289,7 @@ pth_event_t pth_event_concat(pth_event_t evf, ...)
     va_list ap;
 
     if (evf == NULL)
-        return_errno(NULL, EINVAL);
+        return pth_error((pth_event_t)NULL, EINVAL);
 
     /* open ring */
     va_start(ap, evf);
@@ -318,7 +318,7 @@ pth_event_t pth_event_isolate(pth_event_t ev)
     pth_event_t ring;
 
     if (ev == NULL)
-        return_errno(NULL, EINVAL);
+        return pth_error((pth_event_t)NULL, EINVAL);
     ring = NULL;
     if (!(ev->ev_next == ev && ev->ev_prev == ev)) {
         ring = ev->ev_next;
@@ -330,27 +330,27 @@ pth_event_t pth_event_isolate(pth_event_t ev)
     return ring;
 }
 
-/* determine whether the event is occurred */
-int pth_event_occurred(pth_event_t ev)
+/* determine status of the event */
+pth_status_t pth_event_status(pth_event_t ev)
 {
     if (ev == NULL)
-        return_errno(FALSE, EINVAL);
-    return ev->ev_occurred;
+        return pth_error(FALSE, EINVAL);
+    return ev->ev_status;
 }
 
 /* walk to next or previous event in an event ring */
 pth_event_t pth_event_walk(pth_event_t ev, unsigned int direction)
 {
     if (ev == NULL)
-        return_errno(NULL, EINVAL);
+        return pth_error((pth_event_t)NULL, EINVAL);
     do {
         if (direction & PTH_WALK_NEXT)
             ev = ev->ev_next;
         else if (direction & PTH_WALK_PREV)
             ev = ev->ev_prev;
         else
-            return_errno(NULL, EINVAL);
-    } while ((direction & PTH_UNTIL_OCCURRED) && !(ev->ev_occurred));
+            return pth_error((pth_event_t)NULL, EINVAL);
+    } while ((direction & PTH_UNTIL_OCCURRED) && (ev->ev_status != PTH_STATUS_OCCURRED));
     return ev;
 }
 
@@ -361,7 +361,7 @@ int pth_event_free(pth_event_t ev, int mode)
     pth_event_t evn;
 
     if (ev == NULL)
-        return_errno(FALSE, EINVAL);
+        return pth_error(FALSE, EINVAL);
     if (mode == PTH_FREE_THIS) {
         ev->ev_prev->ev_next = ev->ev_next;
         ev->ev_next->ev_prev = ev->ev_prev;
@@ -381,18 +381,18 @@ int pth_event_free(pth_event_t ev, int mode)
 /* wait for one or more events */
 int pth_wait(pth_event_t ev_ring)
 {
-    int occurred;
+    int nonpending;
     pth_event_t ev;
 
     /* at least a waiting ring is required */
     if (ev_ring == NULL)
-        return_errno(-1, EINVAL);
+        return pth_error(-1, EINVAL);
     pth_debug2("pth_wait: enter from thread \"%s\"", pth_current->name);
 
-    /* mark all events in waiting ring as still not occurred */
+    /* mark all events in waiting ring as still pending */
     ev = ev_ring;
     do {
-        ev->ev_occurred = FALSE;
+        ev->ev_status = PTH_STATUS_PENDING;
         pth_debug2("pth_wait: waiting on event 0x%lx", (unsigned long)ev);
         ev = ev->ev_next;
     } while (ev != ev_ring);
@@ -411,19 +411,19 @@ int pth_wait(pth_event_t ev_ring)
     /* unlink event ring from current thread */
     pth_current->events = NULL;
 
-    /* count number of actually occurred events */
+    /* count number of actually occurred (or failed) events */
     ev = ev_ring;
-    occurred = 0;
+    nonpending = 0;
     do {
-        if (ev->ev_occurred) {
-            pth_debug2("pth_wait: occurred event 0x%lx", (unsigned long)ev);
-            occurred++;
+        if (ev->ev_status != PTH_STATUS_PENDING) {
+            pth_debug2("pth_wait: non-pending event 0x%lx", (unsigned long)ev);
+            nonpending++;
         }
         ev = ev->ev_next;
     } while (ev != ev_ring);
 
     /* leave to current thread with number of occurred events */
     pth_debug2("pth_wait: leave to thread \"%s\"", pth_current->name);
-    return occurred;
+    return nonpending;
 }
 
