@@ -1,6 +1,6 @@
 /*
 **  GNU Pth - The GNU Portable Threads
-**  Copyright (c) 1999-2002 Ralf S. Engelschall <rse@engelschall.com>
+**  Copyright (c) 1999-2003 Ralf S. Engelschall <rse@engelschall.com>
 **
 **  This file is part of GNU Pth, a non-preemptive thread scheduling
 **  library which can be found at http://www.gnu.org/software/pth/.
@@ -90,14 +90,14 @@ Sfdisc_t *pth_sfiodisc(void)
     Sfdisc_t *disc;
 
     if ((disc = (Sfdisc_t *)malloc(sizeof(Sfdisc_t))) == NULL)
-        return NULL;
+        return pth_error((SFdisc_t *)NULL, errno);
     disc->readf   = pth_sfio_read;
     disc->writef  = pth_sfio_write;
     disc->seekf   = pth_sfio_seek;
     disc->exceptf = pth_sfio_except;
     return disc;
 #else
-    return_errno(NULL, ENOSYS);
+    return pth_error((Sfdisc_t *)NULL, ENOSYS);
 #endif /* PTH_EXT_SFIO */
 }
 
