@@ -1,6 +1,6 @@
 /*
 **  GNU Pth - The GNU Portable Threads
-**  Copyright (c) 1999-2002 Ralf S. Engelschall <rse@engelschall.com>
+**  Copyright (c) 1999-2003 Ralf S. Engelschall <rse@engelschall.com>
 **
 **  This file is part of GNU Pth, a non-preemptive thread scheduling
 **  library which can be found at http://www.gnu.org/software/pth/.
@@ -43,7 +43,7 @@ int pth_atfork_push(void (*prepare)(void *), void (*parent)(void *),
                     void (*child)(void *), void *arg)
 {
     if (pth_atfork_idx > PTH_ATFORK_MAX-1)
-        return_errno(FALSE, ENOMEM);
+        return pth_error(FALSE, ENOMEM);
     pth_atfork_list[pth_atfork_idx].prepare = prepare;
     pth_atfork_list[pth_atfork_idx].parent  = parent;
     pth_atfork_list[pth_atfork_idx].child   = child;
