@@ -1,6 +1,6 @@
 /*
 **  GNU Pth - The GNU Portable Threads
-**  Copyright (c) 1999-2002 Ralf S. Engelschall <rse@engelschall.com>
+**  Copyright (c) 1999-2003 Ralf S. Engelschall <rse@engelschall.com>
 **
 **  This file is part of GNU Pth, a non-preemptive thread scheduling
 **  library which can be found at http://www.gnu.org/software/pth/.
@@ -35,6 +35,45 @@
 
 #include "pth_p.h"
 
+/* Pth variant of nanosleep(2) */
+int pth_nanosleep(const struct timespec *rqtp, struct timespec *rmtp)
+{
+    pth_time_t until;
+    pth_time_t offset;
+    pth_time_t now;
+    pth_event_t ev;
+    static pth_key_t ev_key = PTH_KEY_INIT;
+
+    /* consistency checks for POSIX conformance */
+    if (rqtp == NULL)
+        return pth_error(-1, EFAULT);
+    if (rqtp->tv_nsec < 0 || rqtp->tv_nsec > (1000*1000000))
+        return pth_error(-1, EINVAL);
+
+    /* short-circuit */
+    if (rqtp->tv_sec == 0 && rqtp->tv_nsec == 0)
+        return 0;
+
+    /* calculate asleep time */
+    offset = pth_time((long)(rqtp->tv_sec), (long)(rqtp->tv_nsec) / 1000);
+    pth_time_set(&until, PTH_TIME_NOW);
+    pth_time_add(&until, &offset);
+
+    /* and let thread sleep until this time is elapsed */
+    ev = pth_event(PTH_EVENT_TIME|PTH_MODE_STATIC, &ev_key, until);
+    pth_wait(ev);
+
+    /* optionally provide amount of slept time */
+    if (rmtp != NULL) {
+        pth_time_set(&now, PTH_TIME_NOW);
+        pth_time_sub(&until, &now);
+        rmtp->tv_sec  = until.tv_sec;
+        rmtp->tv_nsec = until.tv_usec * 1000;
+    }
+
+    return 0;
+}
+
 /* Pth variant of usleep(3) */
 int pth_usleep(unsigned int usec)
 {
@@ -113,7 +152,7 @@ int pth_sigwait_ev(const sigset_t *set, int *sigp, pth_event_t ev_extra)
     int sig;
 
     if (set == NULL || sigp == NULL)
-        return_errno(EINVAL, EINVAL);
+        return pth_error(EINVAL, EINVAL);
 
     /* check whether signal is already pending */
     if (sigpending(&pending) < 0)
@@ -133,8 +172,8 @@ int pth_sigwait_ev(const sigset_t *set, int *sigp, pth_event_t ev_extra)
     pth_wait(ev);
     if (ev_extra != NULL) {
         pth_event_isolate(ev);
-        if (!pth_event_occurred(ev))
-            return_errno(EINTR, EINTR);
+        if (pth_event_status(ev) != PTH_STATUS_OCCURRED)
+            return pth_error(EINTR, EINTR);
     }
 
     /* nothing to do, scheduler has already set *sigp for us */
@@ -152,7 +191,7 @@ pid_t pth_waitpid(pid_t wpid, int *status, int options)
 
     for (;;) {
         /* do a non-blocking poll for the pid */
-        while (   (pid = pth_sc(waitpid)(wpid, status, options|WNOHANG)) < 0 
+        while (   (pid = pth_sc(waitpid)(wpid, status, options|WNOHANG)) < 0
                && errno == EINTR) ;
 
         /* if pid was found or caller requested a polling return immediately */
@@ -195,7 +234,7 @@ int pth_system(const char *cmd)
     /* block SIGCHLD signal */
     sigemptyset(&ss_block);
     sigaddset(&ss_block, SIGCHLD);
-    sigprocmask(SIG_BLOCK, &ss_block, &ss_old);
+    pth_sc(sigprocmask)(SIG_BLOCK, &ss_block, &ss_old);
 
     /* fork the current process */
     pstat = -1;
@@ -207,7 +246,7 @@ int pth_system(const char *cmd)
             /* restore original signal dispositions and execute the command */
             sigaction(SIGINT,  &sa_int,  NULL);
             sigaction(SIGQUIT, &sa_quit, NULL);
-            sigprocmask(SIG_SETMASK, &ss_old, NULL);
+            pth_sc(sigprocmask)(SIG_SETMASK, &ss_old, NULL);
 
             /* stop the Pth scheduling */
             pth_scheduler_kill();
@@ -227,17 +266,17 @@ int pth_system(const char *cmd)
     /* restore original signal dispositions and execute the command */
     sigaction(SIGINT,  &sa_int,  NULL);
     sigaction(SIGQUIT, &sa_quit, NULL);
-    sigprocmask(SIG_SETMASK, &ss_old, NULL);
+    pth_sc(sigprocmask)(SIG_SETMASK, &ss_old, NULL);
 
     /* return error or child process result code */
     return (pid == -1 ? -1 : pstat);
 }
 
 /* Pth variant of select(2) */
-int pth_select(int nfds, fd_set *readfds, fd_set *writefds,
-               fd_set *exceptfds, struct timeval *timeout)
+int pth_select(int nfds, fd_set *rfds, fd_set *wfds,
+               fd_set *efds, struct timeval *timeout)
 {
-    return pth_select_ev(nfds, readfds, writefds, exceptfds, timeout, NULL);
+    return pth_select_ev(nfds, rfds, wfds, efds, timeout, NULL);
 }
 
 /* Pth variant of select(2) with extra events */
@@ -258,17 +297,27 @@ int pth_select_ev(int nfd, fd_set *rfds, fd_set *wfds,
     pth_implicit_init();
     pth_debug2("pth_select_ev: called from thread \"%s\"", pth_current->name);
 
+    /* POSIX.1-2001/SUSv3 compliance */
+    if (nfd < 0 || nfd > FD_SETSIZE)
+        return pth_error(-1, EINVAL);
+    if (timeout != NULL) {
+        if (   timeout->tv_sec  < 0
+            || timeout->tv_usec < 0
+            || timeout->tv_usec >= 1000000 /* a full second */)
+            return pth_error(-1, EINVAL);
+        if (timeout->tv_sec > 31*24*60*60)
+            timeout->tv_sec = 31*24*60*60;
+    }
+
     /* first deal with the special situation of a plain microsecond delay */
     if (nfd == 0 && rfds == NULL && wfds == NULL && efds == NULL && timeout != NULL) {
-        if (timeout->tv_sec < 0 || timeout->tv_usec < 0)
-            return_errno(-1, EINVAL);
-        if (timeout->tv_sec == 0 && timeout->tv_usec < 500000) {
+        if (timeout->tv_sec == 0 && timeout->tv_usec <= 10000 /* 1/100 second */) {
             /* very small delays are acceptable to be performed directly */
             while (   pth_sc(select)(0, NULL, NULL, NULL, timeout) < 0
                    && errno == EINTR) ;
         }
         else {
-            /* larger delays have to use the scheduler */
+            /* larger delays have to go through the scheduler */
             ev = pth_event(PTH_EVENT_TIME|PTH_MODE_STATIC, &ev_key_timeout,
                            pth_timeout(timeout->tv_sec, timeout->tv_usec));
             if (ev_extra != NULL)
@@ -276,60 +325,61 @@ int pth_select_ev(int nfd, fd_set *rfds, fd_set *wfds,
             pth_wait(ev);
             if (ev_extra != NULL) {
                 pth_event_isolate(ev);
-                if (!pth_event_occurred(ev))
-                    return_errno(-1, EINTR);
+                if (pth_event_status(ev) != PTH_STATUS_OCCURRED)
+                    return pth_error(-1, EINTR);
             }
         }
-        /* POSIX compliance */
+        /* POSIX.1-2001/SUSv3 compliance */
         if (rfds != NULL) FD_ZERO(rfds);
         if (wfds != NULL) FD_ZERO(wfds);
         if (efds != NULL) FD_ZERO(efds);
         return 0;
     }
 
-    /* now directly poll filedescriptor sets to avoid unneccessary
+    /* now directly poll filedescriptor sets to avoid unnecessary
        (and resource consuming because of context switches, etc) event
        handling through the scheduler. We've to be carefully here, because not
-       all platforms guarranty us that the sets are unmodified if an error
-       or timeout occured. */
+       all platforms guaranty us that the sets are unmodified if an error
+       or timeout occurred. */
     delay.tv_sec  = 0;
     delay.tv_usec = 0;
     rtmp = NULL;
     if (rfds != NULL) {
-        rspare = *rfds;
+        memcpy(&rspare, rfds, sizeof(fd_set));
         rtmp = &rspare;
     }
     wtmp = NULL;
     if (wfds != NULL) {
-        wspare = *wfds;
+        memcpy(&wspare, wfds, sizeof(fd_set));
         wtmp = &wspare;
     }
     etmp = NULL;
     if (efds != NULL) {
-        espare = *efds;
+        memcpy(&espare, efds, sizeof(fd_set));
         etmp = &espare;
     }
-    while ((rc = pth_sc(select)(nfd, rtmp, wtmp, etmp, &delay)) < 0 && errno == EINTR) ;
-    if (rc > 0) {
+    while ((rc = pth_sc(select)(nfd, rtmp, wtmp, etmp, &delay)) < 0
+           && errno == EINTR)
+        ;
+    if (rc < 0)
+        /* pass-through immediate error */
+        return pth_error(-1, errno);
+    else if (   rc > 0
+             || (   rc == 0
+                 && timeout != NULL
+                 && pth_time_cmp(timeout, PTH_TIME_ZERO) == 0)) {
+        /* pass-through immediate success */
         if (rfds != NULL)
-            *rfds = rspare;
+            memcpy(rfds, &rspare, sizeof(fd_set));
         if (wfds != NULL)
-            *wfds = wspare;
+            memcpy(wfds, &wspare, sizeof(fd_set));
         if (efds != NULL)
-            *efds = espare;
+            memcpy(efds, &espare, sizeof(fd_set));
         return rc;
     }
-    if (rc == 0 && timeout != NULL) {
-        if (pth_time_cmp(timeout, PTH_TIME_ZERO) == 0) {
-            /* POSIX compliance */
-            if (rfds != NULL) FD_ZERO(rfds);
-            if (wfds != NULL) FD_ZERO(wfds);
-            if (efds != NULL) FD_ZERO(efds);
-            return 0;
-        }
-    }
 
-    /* suspend current thread until one fd is ready or the timeout occurred */
+    /* suspend current thread until one filedescriptor
+       is ready or the timeout occurred */
     rc = -1;
     ev = ev_select = pth_event(PTH_EVENT_SELECT|PTH_MODE_STATIC,
                                &ev_key_select, &rc, nfd, rfds, wfds, efds);
@@ -342,27 +392,65 @@ int pth_select_ev(int nfd, fd_set *rfds, fd_set *wfds,
     if (ev_extra != NULL)
         pth_event_concat(ev, ev_extra, NULL);
     pth_wait(ev);
-    selected = FALSE;
-    pth_event_isolate(ev_select);
-    if (pth_event_occurred(ev_select))
-        selected = TRUE;
-    if (timeout != NULL) {
+    if (ev_extra != NULL)
+        pth_event_isolate(ev_extra);
+    if (timeout != NULL)
         pth_event_isolate(ev_timeout);
-        if (pth_event_occurred(ev_timeout)) {
-            selected = TRUE;
-            /* POSIX compliance */
-            if (rfds != NULL) FD_ZERO(rfds);
-            if (wfds != NULL) FD_ZERO(wfds);
-            if (efds != NULL) FD_ZERO(efds);
-            rc = 0;
-        }
+
+    /* select return code semantics */
+    if (pth_event_status(ev_select) == PTH_STATUS_FAILED)
+        return pth_error(-1, EBADF);
+    selected = FALSE;
+    if (pth_event_status(ev_select) == PTH_STATUS_OCCURRED)
+        selected = TRUE;
+    if (   timeout != NULL
+        && pth_event_status(ev_timeout) == PTH_STATUS_OCCURRED) {
+        selected = TRUE;
+        /* POSIX.1-2001/SUSv3 compliance */
+        if (rfds != NULL) FD_ZERO(rfds);
+        if (wfds != NULL) FD_ZERO(wfds);
+        if (efds != NULL) FD_ZERO(efds);
+        rc = 0;
     }
     if (ev_extra != NULL && !selected)
-        return_errno(-1, EINTR);
+        return pth_error(-1, EINTR);
+
     return rc;
 }
 
-/* Pth variant of select(2) */
+/* Pth variant of pth_pselect(2) */
+int pth_pselect(int nfds, fd_set *rfds, fd_set *wfds, fd_set *efds,
+                const struct timespec *ts, const sigset_t *mask)
+{
+    sigset_t omask;
+    struct timeval tv;
+    struct timeval *tvp;
+    int rv;
+
+    /* convert timeout */
+    if (ts != NULL) {
+        tv.tv_sec  = ts->tv_sec;
+        tv.tv_usec = ts->tv_nsec / 1000;
+        tvp = &tv;
+    }
+    else
+        tvp = NULL;
+
+    /* optionally set signal mask */
+    if (mask != NULL)
+        if (pth_sc(sigprocmask)(SIG_SETMASK, mask, &omask) < 0)
+            return pth_error(-1, errno);
+
+    rv = pth_select(nfds, rfds, wfds, efds, tvp);
+
+    /* optionally set signal mask */
+    if (mask != NULL)
+        pth_shield { pth_sc(sigprocmask)(SIG_SETMASK, &omask, NULL); }
+
+    return rv;
+}
+
+/* Pth variant of poll(2) */
 int pth_poll(struct pollfd *pfd, nfds_t nfd, int timeout)
 {
     return pth_poll_ev(pfd, nfd, timeout, NULL);
@@ -373,18 +461,20 @@ int pth_poll(struct pollfd *pfd, nfds_t nfd, int timeout)
            INTERNALLY THE SCHEDULER IS ONLY select(2) BASED!! */
 int pth_poll_ev(struct pollfd *pfd, nfds_t nfd, int timeout, pth_event_t ev_extra)
 {
-    fd_set rfds, wfds, efds;
+    fd_set rfds, wfds, efds, xfds;
     struct timeval tv, *ptv;
-    int maxfd, rc, ok;
+    int maxfd, rc, n;
     unsigned int i;
     char data[64];
 
     pth_implicit_init();
     pth_debug2("pth_poll_ev: called from thread \"%s\"", pth_current->name);
 
-    /* poll(2) semantics */
+    /* argument sanity checks */
     if (pfd == NULL)
-        return_errno(-1, EFAULT);
+        return pth_error(-1, EFAULT);
+    if (nfd < 0 || nfd > FD_SETSIZE)
+        return pth_error(-1, EINVAL);
 
     /* convert timeout number into a timeval structure */
     ptv = &tv;
@@ -399,71 +489,98 @@ int pth_poll_ev(struct pollfd *pfd, nfds_t nfd, int timeout, pth_event_t ev_extr
     }
     else if (timeout > 0) {
         /* return after timeout */
-        ptv->tv_sec  = timeout / 1000;
+        ptv->tv_sec  = (timeout / 1000);
         ptv->tv_usec = (timeout % 1000) * 1000;
     }
     else
-        return_errno(-1, EINVAL);
+        return pth_error(-1, EINVAL);
 
     /* create fd sets and determine max fd */
     maxfd = -1;
     FD_ZERO(&rfds);
     FD_ZERO(&wfds);
     FD_ZERO(&efds);
-    for(i = 0; i < nfd; i++) {
-        if (!pth_util_fd_valid(pfd[i].fd))
-            return_errno(-1, EBADF);
-        if (pfd[i].events & POLLIN)
+    FD_ZERO(&xfds);
+    for (i = 0; i < nfd; i++) {
+        /* convert into fd_sets but remember that BSD select(2) says
+           "the only exceptional condition detectable is out-of-band
+           data received on a socket", hence we push POLLWRBAND events
+           onto wfds instead of efds. Additionally, remember invalid
+           filedescriptors in an extra fd_set xfds. */
+        if (!pth_util_fd_valid(pfd[i].fd)) {
+            FD_SET(pfd[i].fd, &xfds);
+            continue;
+        }
+        if (pfd[i].events & (POLLIN|POLLRDNORM))
             FD_SET(pfd[i].fd, &rfds);
-        if (pfd[i].events & POLLOUT)
+        if (pfd[i].events & (POLLOUT|POLLWRNORM|POLLWRBAND))
             FD_SET(pfd[i].fd, &wfds);
-        if (pfd[i].events & POLLPRI)
+        if (pfd[i].events & (POLLPRI|POLLRDBAND))
             FD_SET(pfd[i].fd, &efds);
-        if (pfd[i].fd >= maxfd && (pfd[i].events & (POLLIN|POLLOUT|POLLPRI)))
+        if (   pfd[i].fd >= maxfd
+            && (pfd[i].events & (POLLIN|POLLOUT|POLLPRI|
+                                 POLLRDNORM|POLLRDBAND|
+                                 POLLWRNORM|POLLWRBAND)))
             maxfd = pfd[i].fd;
     }
-    if (maxfd == -1)
-        return_errno(-1, EINVAL);
 
-    /* examine fd sets */
-    rc = pth_select_ev(maxfd+1, &rfds, &wfds, &efds, ptv, ev_extra);
+    /* examine fd sets with pth_select(3) */
+    rc = -1;
+    if (maxfd != -1) {
+        rc = pth_select_ev(maxfd+1, &rfds, &wfds, &efds, ptv, ev_extra);
+        if (rc < 0)
+            return pth_error(-1, errno);
+        else if (rc == 0)
+            return 0;
+    }
 
-    /* establish results */
-    if (rc > 0) {
-        rc = 0;
-        for (i = 0; i < nfd; i++) {
-            ok = 0;
-            pfd[i].revents = 0;
-            if (pfd[i].fd < 0) {
+    /* POSIX.1-2001/SUSv3 compliant result establishment */
+    n = 0;
+    for (i = 0; i < nfd; i++) {
+        pfd[i].revents = 0;
+        if (FD_ISSET(pfd[i].fd, &xfds)) {
+            if (pfd[i].fd >= 0) {
                 pfd[i].revents |= POLLNVAL;
-                continue;
+                n++;
             }
-            if (FD_ISSET(pfd[i].fd, &rfds)) {
+            continue;
+        }
+        if (maxfd == -1)
+            continue;
+        if (FD_ISSET(pfd[i].fd, &rfds)) {
+            if (pfd[i].events & POLLIN)
                 pfd[i].revents |= POLLIN;
-                ok++;
-                /* support for POLLHUP */
-                if (recv(pfd[i].fd, data, 64, MSG_PEEK) == -1) {
-                    if (   errno == ESHUTDOWN    || errno == ECONNRESET
-                        || errno == ECONNABORTED || errno == ENETRESET) {
-                        pfd[i].revents &= ~(POLLIN);
-                        pfd[i].revents |= POLLHUP;
-                        ok--;
-                    }
-                }
+            if (pfd[i].events & POLLRDNORM)
+                pfd[i].revents |= POLLRDNORM;
+            n++;
+            /* support for POLLHUP */
+            if (   recv(pfd[i].fd, data, sizeof(data), MSG_PEEK) == -1
+                && (   errno == ESHUTDOWN    || errno == ECONNRESET
+                    || errno == ECONNABORTED || errno == ENETRESET    )) {
+                pfd[i].revents &= ~(POLLIN);
+                pfd[i].revents &= ~(POLLRDNORM);
+                pfd[i].revents |= POLLHUP;
             }
-            if (FD_ISSET(pfd[i].fd, &wfds)) {
+        }
+        else if (FD_ISSET(pfd[i].fd, &wfds)) {
+            if (pfd[i].events & POLLOUT)
                 pfd[i].revents |= POLLOUT;
-                ok++;
-            }
-            if (FD_ISSET(pfd[i].fd, &efds)) {
+            if (pfd[i].events & POLLWRNORM)
+                pfd[i].revents |= POLLWRNORM;
+            if (pfd[i].events & POLLWRBAND)
+                pfd[i].revents |= POLLWRBAND;
+            n++;
+        }
+        else if (FD_ISSET(pfd[i].fd, &efds)) {
+            if (pfd[i].events & POLLPRI)
                 pfd[i].revents |= POLLPRI;
-                ok++;
-            }
-            if (ok)
-                rc++;
+            if (pfd[i].events & POLLRDBAND)
+                pfd[i].revents |= POLLRDBAND;
+            n++;
         }
     }
-    return rc;
+
+    return n;
 }
 
 /* Pth variant of connect(2) */
@@ -484,8 +601,13 @@ int pth_connect_ev(int s, const struct sockaddr *addr, socklen_t addrlen, pth_ev
     pth_implicit_init();
     pth_debug2("pth_connect_ev: enter from thread \"%s\"", pth_current->name);
 
+    /* POSIX compliance */
+    if (!pth_util_fd_valid(s))
+        return pth_error(-1, EBADF);
+
     /* force filedescriptor into non-blocking mode */
-    fdmode = pth_fdmode(s, PTH_FDMODE_NONBLOCK);
+    if ((fdmode = pth_fdmode(s, PTH_FDMODE_NONBLOCK)) == PTH_FDMODE_ERROR)
+        return pth_error(-1, EBADF);
 
     /* try to connect */
     while (   (rv = pth_sc(connect)(s, (struct sockaddr *)addr, addrlen)) == -1
@@ -493,7 +615,7 @@ int pth_connect_ev(int s, const struct sockaddr *addr, socklen_t addrlen, pth_ev
         ;
 
     /* restore filedescriptor mode */
-    errno_shield { pth_fdmode(s, fdmode); }
+    pth_shield { pth_fdmode(s, fdmode); }
 
     /* if it is still on progress wait until socket is really writeable */
     if (rv == -1 && errno == EINPROGRESS && fdmode != PTH_FDMODE_NONBLOCK) {
@@ -503,15 +625,15 @@ int pth_connect_ev(int s, const struct sockaddr *addr, socklen_t addrlen, pth_ev
         pth_wait(ev);
         if (ev_extra != NULL) {
             pth_event_isolate(ev);
-            if (!pth_event_occurred(ev))
-                return_errno(-1, EINTR);
+            if (pth_event_status(ev) != PTH_STATUS_OCCURRED)
+                return pth_error(-1, EINTR);
         }
         errlen = sizeof(err);
         if (getsockopt(s, SOL_SOCKET, SO_ERROR, (void *)&err, &errlen) == -1)
             return -1;
         if (err == 0)
             return 0;
-        return_errno(rv, err);
+        return pth_error(rv, err);
     }
 
     pth_debug2("pth_connect_ev: leave to thread \"%s\"", pth_current->name);
@@ -535,8 +657,13 @@ int pth_accept_ev(int s, struct sockaddr *addr, socklen_t *addrlen, pth_event_t
     pth_implicit_init();
     pth_debug2("pth_accept_ev: enter from thread \"%s\"", pth_current->name);
 
+    /* POSIX compliance */
+    if (!pth_util_fd_valid(s))
+        return pth_error(-1, EBADF);
+
     /* force filedescriptor into non-blocking mode */
-    fdmode = pth_fdmode(s, PTH_FDMODE_NONBLOCK);
+    if ((fdmode = pth_fdmode(s, PTH_FDMODE_NONBLOCK)) == PTH_FDMODE_ERROR)
+        return pth_error(-1, EBADF);
 
     /* poll socket via accept */
     ev = NULL;
@@ -554,15 +681,15 @@ int pth_accept_ev(int s, struct sockaddr *addr, socklen_t *addrlen, pth_event_t
         /* check for the extra events */
         if (ev_extra != NULL) {
             pth_event_isolate(ev);
-            if (!pth_event_occurred(ev)) {
+            if (pth_event_status(ev) != PTH_STATUS_OCCURRED) {
                 pth_fdmode(s, fdmode);
-                return_errno(-1, EINTR);
+                return pth_error(-1, EINTR);
             }
         }
     }
 
     /* restore filedescriptor mode */
-    errno_shield {
+    pth_shield {
         pth_fdmode(s, fdmode);
         if (rv != -1)
             pth_fdmode(rv, fdmode);
@@ -585,6 +712,7 @@ ssize_t pth_read_ev(int fd, void *buf, size_t nbytes, pth_event_t ev_extra)
     pth_event_t ev;
     static pth_key_t ev_key = PTH_KEY_INIT;
     fd_set fds;
+    int fdmode;
     int n;
 
     pth_implicit_init();
@@ -593,33 +721,39 @@ ssize_t pth_read_ev(int fd, void *buf, size_t nbytes, pth_event_t ev_extra)
     /* POSIX compliance */
     if (nbytes == 0)
         return 0;
+    if (!pth_util_fd_valid(fd))
+        return pth_error(-1, EBADF);
+
+    /* check mode of filedescriptor */
+    if ((fdmode = pth_fdmode(fd, PTH_FDMODE_POLL)) == PTH_FDMODE_ERROR)
+        return pth_error(-1, EBADF);
 
     /* poll filedescriptor if not already in non-blocking operation */
-    if (pth_fdmode(fd, PTH_FDMODE_POLL) == PTH_FDMODE_BLOCK) {
+    if (fdmode == PTH_FDMODE_BLOCK) {
 
         /* now directly poll filedescriptor for readability
            to avoid unneccessary (and resource consuming because of context
            switches, etc) event handling through the scheduler */
-        if (!pth_util_fd_valid(fd))
-            return_errno(-1, EBADF);
         FD_ZERO(&fds);
         FD_SET(fd, &fds);
         delay.tv_sec  = 0;
         delay.tv_usec = 0;
         while ((n = pth_sc(select)(fd+1, &fds, NULL, NULL, &delay)) < 0
                && errno == EINTR) ;
+        if (n < 0 && (errno == EINVAL || errno == EBADF))
+            return pth_error(-1, errno);
 
         /* if filedescriptor is still not readable,
            let thread sleep until it is or the extra event occurs */
-        if (n < 1) {
+        if (n == 0) {
             ev = pth_event(PTH_EVENT_FD|PTH_UNTIL_FD_READABLE|PTH_MODE_STATIC, &ev_key, fd);
             if (ev_extra != NULL)
                 pth_event_concat(ev, ev_extra, NULL);
             n = pth_wait(ev);
             if (ev_extra != NULL) {
                 pth_event_isolate(ev);
-                if (!pth_event_occurred(ev))
-                    return_errno(-1, EINTR);
+                if (pth_event_status(ev) != PTH_STATUS_OCCURRED)
+                    return pth_error(-1, EINTR);
             }
         }
     }
@@ -660,9 +794,12 @@ ssize_t pth_write_ev(int fd, const void *buf, size_t nbytes, pth_event_t ev_extr
     /* POSIX compliance */
     if (nbytes == 0)
         return 0;
+    if (!pth_util_fd_valid(fd))
+        return pth_error(-1, EBADF);
 
     /* force filedescriptor into non-blocking mode */
-    fdmode = pth_fdmode(fd, PTH_FDMODE_NONBLOCK);
+    if ((fdmode = pth_fdmode(fd, PTH_FDMODE_NONBLOCK)) == PTH_FDMODE_ERROR)
+        return pth_error(-1, EBADF);
 
     /* poll filedescriptor if not already in non-blocking operation */
     if (fdmode != PTH_FDMODE_NONBLOCK) {
@@ -670,16 +807,14 @@ ssize_t pth_write_ev(int fd, const void *buf, size_t nbytes, pth_event_t ev_extr
         /* now directly poll filedescriptor for writeability
            to avoid unneccessary (and resource consuming because of context
            switches, etc) event handling through the scheduler */
-        if (!pth_util_fd_valid(fd)) {
-            pth_fdmode(fd, fdmode);
-            return_errno(-1, EBADF);
-        }
         FD_ZERO(&fds);
         FD_SET(fd, &fds);
         delay.tv_sec  = 0;
         delay.tv_usec = 0;
         while ((n = pth_sc(select)(fd+1, NULL, &fds, NULL, &delay)) < 0
                && errno == EINTR) ;
+        if (n < 0 && (errno == EINVAL || errno == EBADF))
+            return pth_error(-1, errno);
 
         rv = 0;
         for (;;) {
@@ -692,9 +827,9 @@ ssize_t pth_write_ev(int fd, const void *buf, size_t nbytes, pth_event_t ev_extr
                 pth_wait(ev);
                 if (ev_extra != NULL) {
                     pth_event_isolate(ev);
-                    if (!pth_event_occurred(ev)) {
+                    if (pth_event_status(ev) != PTH_STATUS_OCCURRED) {
                         pth_fdmode(fd, fdmode);
-                        return_errno(-1, EINTR);
+                        return pth_error(-1, EINTR);
                     }
                 }
             }
@@ -730,7 +865,7 @@ ssize_t pth_write_ev(int fd, const void *buf, size_t nbytes, pth_event_t ev_extr
     }
 
     /* restore filedescriptor mode */
-    errno_shield { pth_fdmode(fd, fdmode); }
+    pth_shield { pth_fdmode(fd, fdmode); }
 
     pth_debug2("pth_write_ev: leave to thread \"%s\"", pth_current->name);
     return rv;
@@ -749,6 +884,7 @@ ssize_t pth_readv_ev(int fd, const struct iovec *iov, int iovcnt, pth_event_t ev
     pth_event_t ev;
     static pth_key_t ev_key = PTH_KEY_INIT;
     fd_set fds;
+    int fdmode;
     int n;
 
     pth_implicit_init();
@@ -756,16 +892,20 @@ ssize_t pth_readv_ev(int fd, const struct iovec *iov, int iovcnt, pth_event_t ev
 
     /* POSIX compliance */
     if (iovcnt <= 0 || iovcnt > UIO_MAXIOV)
-        return_errno(-1, EINVAL);
+        return pth_error(-1, EINVAL);
+    if (!pth_util_fd_valid(fd))
+        return pth_error(-1, EBADF);
+
+    /* check mode of filedescriptor */
+    if ((fdmode = pth_fdmode(fd, PTH_FDMODE_POLL)) == PTH_FDMODE_ERROR)
+        return pth_error(-1, EBADF);
 
     /* poll filedescriptor if not already in non-blocking operation */
-    if (pth_fdmode(fd, PTH_FDMODE_POLL) == PTH_FDMODE_BLOCK) {
+    if (fdmode == PTH_FDMODE_BLOCK) {
 
         /* first directly poll filedescriptor for readability
            to avoid unneccessary (and resource consuming because of context
            switches, etc) event handling through the scheduler */
-        if (!pth_util_fd_valid(fd))
-            return_errno(-1, EBADF);
         FD_ZERO(&fds);
         FD_SET(fd, &fds);
         delay.tv_sec  = 0;
@@ -782,8 +922,8 @@ ssize_t pth_readv_ev(int fd, const struct iovec *iov, int iovcnt, pth_event_t ev
             n = pth_wait(ev);
             if (ev_extra != NULL) {
                 pth_event_isolate(ev);
-                if (!pth_event_occurred(ev))
-                    return_errno(-1, EINTR);
+                if (pth_event_status(ev) != PTH_STATUS_OCCURRED)
+                    return pth_error(-1, EINTR);
             }
         }
     }
@@ -816,11 +956,11 @@ intern ssize_t pth_readv_faked(int fd, const struct iovec *iov, int iovcnt)
     bytes = 0;
     for (i = 0; i < iovcnt; i++) {
         if (iov[i].iov_len <= 0)
-            return_errno((ssize_t)(-1), EINVAL);
+            return pth_error((ssize_t)(-1), EINVAL);
         bytes += iov[i].iov_len;
     }
     if (bytes <= 0)
-        return_errno((ssize_t)(-1), EINVAL);
+        return pth_error((ssize_t)(-1), EINVAL);
 
     /* allocate a temporary buffer */
     if ((buffer = (char *)malloc(bytes)) == NULL)
@@ -843,7 +983,7 @@ intern ssize_t pth_readv_faked(int fd, const struct iovec *iov, int iovcnt)
     }
 
     /* remove the temporary buffer */
-    errno_shield { free(buffer); }
+    pth_shield { free(buffer); }
 
     /* return number of read bytes */
     return(rv);
@@ -878,10 +1018,13 @@ ssize_t pth_writev_ev(int fd, const struct iovec *iov, int iovcnt, pth_event_t e
 
     /* POSIX compliance */
     if (iovcnt <= 0 || iovcnt > UIO_MAXIOV)
-        return_errno(-1, EINVAL);
+        return pth_error(-1, EINVAL);
+    if (!pth_util_fd_valid(fd))
+        return pth_error(-1, EBADF);
 
     /* force filedescriptor into non-blocking mode */
-    fdmode = pth_fdmode(fd, PTH_FDMODE_NONBLOCK);
+    if ((fdmode = pth_fdmode(fd, PTH_FDMODE_NONBLOCK)) == PTH_FDMODE_ERROR)
+        return pth_error(-1, EBADF);
 
     /* poll filedescriptor if not already in non-blocking operation */
     if (fdmode != PTH_FDMODE_NONBLOCK) {
@@ -889,7 +1032,7 @@ ssize_t pth_writev_ev(int fd, const struct iovec *iov, int iovcnt, pth_event_t e
         if (iovcnt > sizeof(tiov_stack)) {
             tiovcnt = (sizeof(struct iovec) * UIO_MAXIOV);
             if ((tiov = (struct iovec *)malloc(tiovcnt)) == NULL)
-                return -1 /* errno is set */;
+                return pth_error(-1, errno);
         }
         else {
             tiovcnt = sizeof(tiov_stack);
@@ -908,12 +1051,6 @@ ssize_t pth_writev_ev(int fd, const struct iovec *iov, int iovcnt, pth_event_t e
         /* first directly poll filedescriptor for writeability
            to avoid unneccessary (and resource consuming because of context
            switches, etc) event handling through the scheduler */
-        if (!pth_util_fd_valid(fd)) {
-            pth_fdmode(fd, fdmode);
-            if (iovcnt > sizeof(tiov_stack))
-                free(tiov);
-            return_errno(-1, EBADF);
-        }
         FD_ZERO(&fds);
         FD_SET(fd, &fds);
         delay.tv_sec  = 0;
@@ -931,11 +1068,11 @@ ssize_t pth_writev_ev(int fd, const struct iovec *iov, int iovcnt, pth_event_t e
                 pth_wait(ev);
                 if (ev_extra != NULL) {
                     pth_event_isolate(ev);
-                    if (!pth_event_occurred(ev)) {
+                    if (pth_event_status(ev) != PTH_STATUS_OCCURRED) {
                         pth_fdmode(fd, fdmode);
                         if (iovcnt > sizeof(tiov_stack))
                             free(tiov);
-                        return_errno(-1, EINTR);
+                        return pth_error(-1, EINTR);
                     }
                 }
             }
@@ -985,7 +1122,7 @@ ssize_t pth_writev_ev(int fd, const struct iovec *iov, int iovcnt, pth_event_t e
     }
 
     /* restore filedescriptor mode */
-    errno_shield { pth_fdmode(fd, fdmode); }
+    pth_shield { pth_fdmode(fd, fdmode); }
 
     pth_debug2("pth_writev_ev: leave to thread \"%s\"", pth_current->name);
     return rv;
@@ -1008,7 +1145,7 @@ intern ssize_t pth_writev_iov_bytes(const struct iovec *iov, int iovcnt)
 
 /* advance the virtual pointer of a struct iov */
 intern void pth_writev_iov_advance(const struct iovec *riov, int riovcnt, size_t advance,
-                                   struct iovec **liov, int *liovcnt, 
+                                   struct iovec **liov, int *liovcnt,
                                    struct iovec *tiov, int tiovcnt)
 {
     int i;
@@ -1055,11 +1192,11 @@ intern ssize_t pth_writev_faked(int fd, const struct iovec *iov, int iovcnt)
     bytes = 0;
     for (i = 0; i < iovcnt; i++) {
         if (iov[i].iov_len <= 0)
-            return_errno((ssize_t)(-1), EINVAL);
+            return pth_error((ssize_t)(-1), EINVAL);
         bytes += iov[i].iov_len;
     }
     if (bytes <= 0)
-        return_errno((ssize_t)(-1), EINVAL);
+        return pth_error((ssize_t)(-1), EINVAL);
 
     /* allocate a temporary buffer to hold the data */
     if ((buffer = (char *)malloc(bytes)) == NULL)
@@ -1080,7 +1217,7 @@ intern ssize_t pth_writev_faked(int fd, const struct iovec *iov, int iovcnt)
     rv = pth_sc(write)(fd, buffer, bytes);
 
     /* remove the temporary buffer */
-    errno_shield { free(buffer); }
+    pth_shield { free(buffer); }
 
     return(rv);
 }
@@ -1111,7 +1248,7 @@ ssize_t pth_pread(int fd, void *buf, size_t nbytes, off_t offset)
     rc = pth_read(fd, buf, nbytes);
 
     /* restore the old offset situation */
-    errno_shield { lseek(fd, old_offset, SEEK_SET); }
+    pth_shield { lseek(fd, old_offset, SEEK_SET); }
 
     /* unprotect and return result of read */
     pth_mutex_release(&mutex);
@@ -1144,7 +1281,7 @@ ssize_t pth_pwrite(int fd, const void *buf, size_t nbytes, off_t offset)
     rc = pth_write(fd, buf, nbytes);
 
     /* restore the old offset situation */
-    errno_shield { lseek(fd, old_offset, SEEK_SET); }
+    pth_shield { lseek(fd, old_offset, SEEK_SET); }
 
     /* unprotect and return result of write */
     pth_mutex_release(&mutex);
@@ -1176,6 +1313,7 @@ ssize_t pth_recvfrom_ev(int fd, void *buf, size_t nbytes, int flags, struct sock
     pth_event_t ev;
     static pth_key_t ev_key = PTH_KEY_INIT;
     fd_set fds;
+    int fdmode;
     int n;
 
     pth_implicit_init();
@@ -1184,33 +1322,41 @@ ssize_t pth_recvfrom_ev(int fd, void *buf, size_t nbytes, int flags, struct sock
     /* POSIX compliance */
     if (nbytes == 0)
         return 0;
+    if (!pth_util_fd_valid(fd))
+        return pth_error(-1, EBADF);
+
+    /* check mode of filedescriptor */
+    if ((fdmode = pth_fdmode(fd, PTH_FDMODE_POLL)) == PTH_FDMODE_ERROR)
+        return pth_error(-1, EBADF);
 
     /* poll filedescriptor if not already in non-blocking operation */
-    if (pth_fdmode(fd, PTH_FDMODE_POLL) == PTH_FDMODE_BLOCK) {
+    if (fdmode == PTH_FDMODE_BLOCK) {
 
         /* now directly poll filedescriptor for readability
            to avoid unneccessary (and resource consuming because of context
            switches, etc) event handling through the scheduler */
         if (!pth_util_fd_valid(fd))
-            return_errno(-1, EBADF);
+            return pth_error(-1, EBADF);
         FD_ZERO(&fds);
         FD_SET(fd, &fds);
         delay.tv_sec  = 0;
         delay.tv_usec = 0;
         while ((n = pth_sc(select)(fd+1, &fds, NULL, NULL, &delay)) < 0
                && errno == EINTR) ;
+        if (n < 0 && (errno == EINVAL || errno == EBADF))
+            return pth_error(-1, errno);
 
         /* if filedescriptor is still not readable,
            let thread sleep until it is or the extra event occurs */
-        if (n < 1) {
+        if (n == 0) {
             ev = pth_event(PTH_EVENT_FD|PTH_UNTIL_FD_READABLE|PTH_MODE_STATIC, &ev_key, fd);
             if (ev_extra != NULL)
                 pth_event_concat(ev, ev_extra, NULL);
             n = pth_wait(ev);
             if (ev_extra != NULL) {
                 pth_event_isolate(ev);
-                if (!pth_event_occurred(ev))
-                    return_errno(-1, EINTR);
+                if (pth_event_status(ev) != PTH_STATUS_OCCURRED)
+                    return pth_error(-1, EINTR);
             }
         }
     }
@@ -1263,9 +1409,12 @@ ssize_t pth_sendto_ev(int fd, const void *buf, size_t nbytes, int flags, const s
     /* POSIX compliance */
     if (nbytes == 0)
         return 0;
+    if (!pth_util_fd_valid(fd))
+        return pth_error(-1, EBADF);
 
     /* force filedescriptor into non-blocking mode */
-    fdmode = pth_fdmode(fd, PTH_FDMODE_NONBLOCK);
+    if ((fdmode = pth_fdmode(fd, PTH_FDMODE_NONBLOCK)) == PTH_FDMODE_ERROR)
+        return pth_error(-1, EBADF);
 
     /* poll filedescriptor if not already in non-blocking operation */
     if (fdmode != PTH_FDMODE_NONBLOCK) {
@@ -1275,7 +1424,7 @@ ssize_t pth_sendto_ev(int fd, const void *buf, size_t nbytes, int flags, const s
            switches, etc) event handling through the scheduler */
         if (!pth_util_fd_valid(fd)) {
             pth_fdmode(fd, fdmode);
-            return_errno(-1, EBADF);
+            return pth_error(-1, EBADF);
         }
         FD_ZERO(&fds);
         FD_SET(fd, &fds);
@@ -1283,21 +1432,23 @@ ssize_t pth_sendto_ev(int fd, const void *buf, size_t nbytes, int flags, const s
         delay.tv_usec = 0;
         while ((n = pth_sc(select)(fd+1, NULL, &fds, NULL, &delay)) < 0
                && errno == EINTR) ;
+        if (n < 0 && (errno == EINVAL || errno == EBADF))
+            return pth_error(-1, errno);
 
         rv = 0;
         for (;;) {
             /* if filedescriptor is still not writeable,
                let thread sleep until it is or event occurs */
-            if (n < 1) {
+            if (n == 0) {
                 ev = pth_event(PTH_EVENT_FD|PTH_UNTIL_FD_WRITEABLE|PTH_MODE_STATIC, &ev_key, fd);
                 if (ev_extra != NULL)
                     pth_event_concat(ev, ev_extra, NULL);
                 pth_wait(ev);
                 if (ev_extra != NULL) {
                     pth_event_isolate(ev);
-                    if (!pth_event_occurred(ev)) {
+                    if (pth_event_status(ev) != PTH_STATUS_OCCURRED) {
                         pth_fdmode(fd, fdmode);
-                        return_errno(-1, EINTR);
+                        return pth_error(-1, EINTR);
                     }
                 }
             }
@@ -1333,7 +1484,7 @@ ssize_t pth_sendto_ev(int fd, const void *buf, size_t nbytes, int flags, const s
     }
 
     /* restore filedescriptor mode */
-    errno_shield { pth_fdmode(fd, fdmode); }
+    pth_shield { pth_fdmode(fd, fdmode); }
 
     pth_debug2("pth_sendto_ev: leave to thread \"%s\"", pth_current->name);
     return rv;
