@@ -1,6 +1,6 @@
 /*
 **  GNU Pth - The GNU Portable Threads
-**  Copyright (c) 1999-2002 Ralf S. Engelschall <rse@engelschall.com>
+**  Copyright (c) 1999-2003 Ralf S. Engelschall <rse@engelschall.com>
 **
 **  This file is part of GNU Pth, a non-preemptive thread scheduling
 **  library which can be found at http://www.gnu.org/software/pth/.
@@ -43,6 +43,18 @@ intern int pth_initialized = FALSE;
         pth_init();
 #endif
 
+#ifdef PTH_EX
+/* exception handling callback functions */
+static ex_ctx_t *pth_ex_ctx(void)
+{
+    return &(pth_current->ex_ctx);
+}
+static void pth_ex_terminate(ex_t *ex)
+{
+    pth_exit(ex->ex_value);
+}
+#endif
+
 /* initialize the package */
 int pth_init(void)
 {
@@ -51,14 +63,26 @@ int pth_init(void)
     /* support for implicit initialization calls
        and to prevent multiple explict initialization, too */
     if (pth_initialized)
-        return_errno(FALSE, EPERM);
+        return pth_error(FALSE, EPERM);
     else
         pth_initialized = TRUE;
 
     pth_debug1("pth_init: enter");
 
+    /* initialize syscall wrapping */
+    pth_syscall_init();
+
     /* initialize the scheduler */
-    pth_scheduler_init();
+    if (!pth_scheduler_init()) {
+        pth_shield { pth_syscall_kill(); }
+        return pth_error(FALSE, EAGAIN);
+    }
+
+#ifdef PTH_EX
+    /* optional support for exceptional handling */
+    __ex_ctx       = pth_ex_ctx;
+    __ex_terminate = pth_ex_terminate;
+#endif
 
     /* spawn the scheduler thread */
     t_attr = pth_attr_new();
@@ -70,9 +94,10 @@ int pth_init(void)
     pth_attr_set(t_attr, PTH_ATTR_STACK_ADDR,   NULL);
     pth_sched = pth_spawn(t_attr, pth_scheduler, NULL);
     if (pth_sched == NULL) {
-        errno_shield { 
+        pth_shield {
             pth_attr_destroy(t_attr);
             pth_scheduler_kill();
+            pth_syscall_kill();
         }
         return FALSE;
     }
@@ -86,9 +111,10 @@ int pth_init(void)
     pth_attr_set(t_attr, PTH_ATTR_STACK_ADDR,   NULL);
     pth_main = pth_spawn(t_attr, (void *(*)(void *))(-1), NULL);
     if (pth_main == NULL) {
-        errno_shield { 
+        pth_shield {
             pth_attr_destroy(t_attr);
             pth_scheduler_kill();
+            pth_syscall_kill();
         }
         return FALSE;
     }
@@ -112,14 +138,19 @@ int pth_init(void)
 /* kill the package internals */
 int pth_kill(void)
 {
-    if (pth_current != pth_main) 
-        return_errno(FALSE, EPERM);
+    if (pth_current != pth_main)
+        return pth_error(FALSE, EPERM);
     pth_debug1("pth_kill: enter");
     pth_thread_cleanup(pth_main);
     pth_scheduler_kill();
     pth_initialized = FALSE;
     pth_tcb_free(pth_sched);
     pth_tcb_free(pth_main);
+    pth_syscall_kill();
+#ifdef PTH_EX
+    __ex_ctx       = __ex_ctx_default;
+    __ex_terminate = __ex_terminate_default;
+#endif
     pth_debug1("pth_kill: leave");
     return TRUE;
 }
@@ -141,7 +172,7 @@ long pth_ctrl(unsigned long query, ...)
             rc += 1; /* pth_current only */
         if (query & PTH_CTRL_GETTHREADS_WAITING)
             rc += pth_pqueue_elements(&pth_WQ);
-        if (query & PTH_CTRL_GETTHREADS_SUSPENDED) 
+        if (query & PTH_CTRL_GETTHREADS_SUSPENDED)
             rc += pth_pqueue_elements(&pth_SQ);
         if (query & PTH_CTRL_GETTHREADS_DEAD)
             rc += pth_pqueue_elements(&pth_DQ);
@@ -166,7 +197,7 @@ long pth_ctrl(unsigned long query, ...)
         rc = -1;
     va_end(ap);
     if (rc == -1)
-        return_errno(-1, EINVAL);
+        return pth_error(-1, EINVAL);
     return rc;
 }
 
@@ -177,9 +208,11 @@ static void pth_spawn_trampoline(void)
 
     /* just jump into the start routine */
     data = (*pth_current->start_func)(pth_current->start_arg);
-    /* and do an implicit exit of the tread with the result value */
+
+    /* and do an implicit exit of the thread with the result value */
     pth_exit(data);
-    /* no return! */
+
+    /* NOTREACHED */
     abort();
 }
 pth_t pth_spawn(pth_attr_t attr, void *(*func)(void *), void *arg)
@@ -193,7 +226,7 @@ pth_t pth_spawn(pth_attr_t attr, void *(*func)(void *), void *arg)
 
     /* consistency */
     if (func == NULL)
-        return_errno(NULL, EINVAL);
+        return pth_error((pth_t)NULL, EINVAL);
 
     /* support the special case of main() */
     if (func == (void *(*)(void *))(-1))
@@ -201,9 +234,9 @@ pth_t pth_spawn(pth_attr_t attr, void *(*func)(void *), void *arg)
 
     /* allocate a new thread control block */
     stacksize = (attr == PTH_ATTR_DEFAULT ? 64*1024 : attr->a_stacksize);
-    stackaddr = (attr == PTH_ATTR_DEFAULT ? NULL : attr->a_stackaddr);
+    stackaddr = (attr == PTH_ATTR_DEFAULT ? NULL    : attr->a_stackaddr);
     if ((t = pth_tcb_alloc(stacksize, stackaddr)) == NULL)
-        return NULL; /* errno is inherited */
+        return pth_error((pth_t)NULL, errno);
 
     /* configure remaining attributes */
     if (attr != PTH_ATTR_DEFAULT) {
@@ -211,6 +244,7 @@ pth_t pth_spawn(pth_attr_t attr, void *(*func)(void *), void *arg)
         t->prio        = attr->a_prio;
         t->joinable    = attr->a_joinable;
         t->cancelstate = attr->a_cancelstate;
+        t->dispatches  = attr->a_dispatches;
         pth_util_cpystrn(t->name, attr->a_name, PTH_TCB_NAMELEN);
     }
     else if (pth_current != NULL) {
@@ -218,8 +252,9 @@ pth_t pth_spawn(pth_attr_t attr, void *(*func)(void *), void *arg)
         t->prio        = pth_current->prio;
         t->joinable    = pth_current->joinable;
         t->cancelstate = pth_current->cancelstate;
-        pth_snprintf(t->name, PTH_TCB_NAMELEN, "%s.child@%d=0x%lx", 
-                     pth_current->name, (unsigned int)time(NULL), 
+        t->dispatches  = 0;
+        pth_snprintf(t->name, PTH_TCB_NAMELEN, "%s.child@%d=0x%lx",
+                     pth_current->name, (unsigned int)time(NULL),
                      (unsigned long)pth_current);
     }
     else {
@@ -227,6 +262,7 @@ pth_t pth_spawn(pth_attr_t attr, void *(*func)(void *), void *arg)
         t->prio        = PTH_PRIO_STD;
         t->joinable    = TRUE;
         t->cancelstate = PTH_CANCEL_DEFAULT;
+        t->dispatches  = 0;
         pth_snprintf(t->name, PTH_TCB_NAMELEN,
                      "user/%x", (unsigned int)time(NULL));
     }
@@ -255,19 +291,24 @@ pth_t pth_spawn(pth_attr_t attr, void *(*func)(void *), void *arg)
     t->data_value = NULL;
     t->data_count = 0;
 
-    /* initialize cancellaton stuff */
+    /* initialize cancellation stuff */
     t->cancelreq   = FALSE;
     t->cleanups    = NULL;
 
     /* initialize mutex stuff */
     pth_ring_init(&t->mutexring);
 
+#ifdef PTH_EX
+    /* initialize exception handling context */
+    EX_CTX_INITIALIZE(&t->ex_ctx);
+#endif
+
     /* initialize the machine context of this new thread */
     if (t->stacksize > 0) { /* the "main thread" (indicated by == 0) is special! */
         if (!pth_mctx_set(&t->mctx, pth_spawn_trampoline,
                           t->stack, ((char *)t->stack+t->stacksize))) {
-            errno_shield { pth_tcb_free(t); }
-            return NULL;
+            pth_shield { pth_tcb_free(t); }
+            return pth_error((pth_t)NULL, errno);
         }
     }
 
@@ -297,7 +338,7 @@ int pth_raise(pth_t t, int sig)
     struct sigaction sa;
 
     if (t == NULL || t == pth_current || (sig < 0 || sig > PTH_NSIG))
-        return_errno(FALSE, EINVAL);
+        return pth_error(FALSE, EINVAL);
     if (sig == 0)
         /* just test whether thread exists */
         return pth_thread_exists(t);
@@ -324,7 +365,7 @@ intern int pth_thread_exists(pth_t t)
             if (!pth_pqueue_contains(&pth_WQ, t))
                 if (!pth_pqueue_contains(&pth_SQ, t))
                     if (!pth_pqueue_contains(&pth_DQ, t))
-                        return_errno(FALSE, ESRCH); /* not found */
+                        return pth_error(FALSE, ESRCH); /* not found */
     return TRUE;
 }
 
@@ -345,19 +386,28 @@ intern void pth_thread_cleanup(pth_t thread)
     return;
 }
 
-/* terminates the current thread */
+/* terminate the current thread */
 static int pth_exit_cb(void *arg)
 {
     int rc;
 
-    /* NOTICE: THIS FUNCTION EXECUTES
-       FROM WITHIN THE SCHEDULER THREAD! */
+    /* BE CAREFUL HERE: THIS FUNCTION EXECUTES
+       FROM WITHIN THE _SCHEDULER_ THREAD! */
+
+    /* calculate number of still existing threads in system. Only
+       skipped queue is pth_DQ (dead queue). This queue does not
+       count here, because those threads are non-detached but already
+       terminated ones -- and if we are the only remaining thread (which
+       also wants to terminate and not join those threads) we can signal
+       us through the scheduled event (for which we are running as the
+       test function inside the scheduler) that the whole process can
+       terminate now. */
     rc = 0;
     rc += pth_pqueue_elements(&pth_NQ);
     rc += pth_pqueue_elements(&pth_RQ);
     rc += pth_pqueue_elements(&pth_WQ);
     rc += pth_pqueue_elements(&pth_SQ);
-    rc += pth_pqueue_elements(&pth_DQ);
+
     if (rc == 1 /* just our main thread */)
         return TRUE;
     else
@@ -369,41 +419,43 @@ void pth_exit(void *value)
 
     pth_debug2("pth_exit: marking thread \"%s\" as dead", pth_current->name);
 
-    /* main thread is special:
-       wait until it is the last thread */
+    /* the main thread is special, because its termination
+       would terminate the whole process, so we have to delay 
+       its termination until it is really the last thread */
     if (pth_current == pth_main) {
-        ev = pth_event(PTH_EVENT_FUNC, pth_exit_cb);
-        pth_wait(ev);
-        pth_event_free(ev, PTH_FREE_THIS);
+        if (!pth_exit_cb(NULL)) {
+            ev = pth_event(PTH_EVENT_FUNC, pth_exit_cb);
+            pth_wait(ev);
+            pth_event_free(ev, PTH_FREE_THIS);
+        }
     }
 
     /* execute cleanups */
     pth_thread_cleanup(pth_current);
 
-    /* mark the current thread as dead, so the scheduler removes us */
-    pth_current->join_arg = value;
-    pth_current->state = PTH_STATE_DEAD;
-
     if (pth_current != pth_main) {
         /*
-         * Now we explicitly switch into the scheduler and let it
-         * reap the current thread structure; we can't free it here,
-         * or we'd be running on a stack which malloc() regards as
-         * free memory, which would be a somewhat perilous situation.
+         * Now mark the current thread as dead, explicitly switch into the
+         * scheduler and let it reap the current thread structure; we can't
+         * free it here, or we'd be running on a stack which malloc() regards
+         * as free memory, which would be a somewhat perilous situation.
          */
+        pth_current->join_arg = value;
+        pth_current->state = PTH_STATE_DEAD;
         pth_debug2("pth_exit: switching from thread \"%s\" to scheduler", pth_current->name);
         pth_mctx_switch(&pth_current->mctx, &pth_sched->mctx);
-        abort(); /* not reached! */
     }
     else {
-        /* 
-         * main thread is special: exit the _process_ 
-         * [double-cast to avoid warnings because of size] 
-         */ 
+        /*
+         * main thread is special: exit the _process_
+         * [double-casted to avoid warnings because of size]
+         */
         pth_kill();
         exit((int)((long)value));
-        abort(); /* not reached! */
     }
+
+    /* NOTREACHED */
+    abort();
 }
 
 /* waits for the termination of the specified thread */
@@ -414,11 +466,11 @@ int pth_join(pth_t tid, void **value)
 
     pth_debug2("pth_join: joining thread \"%s\"", tid == NULL ? "-ANY-" : tid->name);
     if (tid == pth_current)
-        return_errno(FALSE, EDEADLK);
+        return pth_error(FALSE, EDEADLK);
     if (tid != NULL && !tid->joinable)
-        return_errno(FALSE, EINVAL);
+        return pth_error(FALSE, EINVAL);
     if (pth_ctrl(PTH_CTRL_GETTHREADS) == 1)
-        return_errno(FALSE, EDEADLK);
+        return pth_error(FALSE, EDEADLK);
     if (tid == NULL)
         tid = pth_pqueue_head(&pth_DQ);
     if (tid == NULL || (tid != NULL && tid->state != PTH_STATE_DEAD)) {
@@ -428,7 +480,7 @@ int pth_join(pth_t tid, void **value)
     if (tid == NULL)
         tid = pth_pqueue_head(&pth_DQ);
     if (tid == NULL || (tid != NULL && tid->state != PTH_STATE_DEAD))
-        return_errno(FALSE, EIO);
+        return pth_error(FALSE, EIO);
     if (value != NULL)
         *value = tid->join_arg;
     pth_pqueue_delete(&pth_DQ, tid);
@@ -451,7 +503,7 @@ int pth_yield(pth_t to)
             default:               q = NULL;
         }
         if (q == NULL || !pth_pqueue_contains(q, to))
-            return_errno(FALSE, EINVAL);
+            return pth_error(FALSE, EINVAL);
     }
 
     /* give a favored thread maximum priority in his queue */
@@ -477,9 +529,9 @@ int pth_suspend(pth_t t)
     pth_pqueue_t *q;
 
     if (t == NULL)
-        return_errno(FALSE, EINVAL);
+        return pth_error(FALSE, EINVAL);
     if (t == pth_sched || t == pth_current)
-        return_errno(FALSE, EPERM);
+        return pth_error(FALSE, EPERM);
     switch (t->state) {
         case PTH_STATE_NEW:     q = &pth_NQ; break;
         case PTH_STATE_READY:   q = &pth_RQ; break;
@@ -487,9 +539,9 @@ int pth_suspend(pth_t t)
         default:                q = NULL;
     }
     if (q == NULL)
-        return_errno(FALSE, EPERM);
+        return pth_error(FALSE, EPERM);
     if (!pth_pqueue_contains(q, t))
-        return_errno(FALSE, ESRCH);
+        return pth_error(FALSE, ESRCH);
     pth_pqueue_delete(q, t);
     pth_pqueue_insert(&pth_SQ, PTH_PRIO_STD, t);
     pth_debug2("pth_suspend: suspend thread \"%s\"\n", t->name);
@@ -502,11 +554,11 @@ int pth_resume(pth_t t)
     pth_pqueue_t *q;
 
     if (t == NULL)
-        return_errno(FALSE, EINVAL);
+        return pth_error(FALSE, EINVAL);
     if (t == pth_sched || t == pth_current)
-        return_errno(FALSE, EPERM);
+        return pth_error(FALSE, EPERM);
     if (!pth_pqueue_contains(&pth_SQ, t))
-        return_errno(FALSE, EPERM);
+        return pth_error(FALSE, EPERM);
     pth_pqueue_delete(&pth_SQ, t);
     switch (t->state) {
         case PTH_STATE_NEW:     q = &pth_NQ; break;
@@ -525,7 +577,7 @@ int pth_fdmode(int fd, int newmode)
     int fdmode;
     int oldmode;
 
-    /* retrieve old mode (usually cheap) */
+    /* retrieve old mode (usually a very cheap operation) */
     if ((fdmode = fcntl(fd, F_GETFL, NULL)) == -1)
         oldmode = PTH_FDMODE_ERROR;
     else if (fdmode & O_NONBLOCKING)
@@ -533,7 +585,7 @@ int pth_fdmode(int fd, int newmode)
     else
         oldmode = PTH_FDMODE_BLOCK;
 
-    /* set new mode (usually expensive) */
+    /* set new mode (usually a more expensive operation) */
     if (oldmode == PTH_FDMODE_BLOCK && newmode == PTH_FDMODE_NONBLOCK)
         fcntl(fd, F_SETFL, (fdmode | O_NONBLOCKING));
     if (oldmode == PTH_FDMODE_NONBLOCK && newmode == PTH_FDMODE_BLOCK)
@@ -551,7 +603,7 @@ int pth_nap(pth_time_t naptime)
     static pth_key_t ev_key = PTH_KEY_INIT;
 
     if (pth_time_cmp(&naptime, PTH_TIME_ZERO) == 0)
-        return_errno(FALSE, EINVAL);
+        return pth_error(FALSE, EINVAL);
     pth_time_set(&until, PTH_TIME_NOW);
     pth_time_add(&until, &naptime);
     ev = pth_event(PTH_EVENT_TIME|PTH_MODE_STATIC, &ev_key, until);
@@ -563,7 +615,7 @@ int pth_nap(pth_time_t naptime)
 int pth_once(pth_once_t *oncectrl, void (*constructor)(void *), void *arg)
 {
     if (oncectrl == NULL || constructor == NULL)
-        return_errno(FALSE, EINVAL);
+        return pth_error(FALSE, EINVAL);
     if (*oncectrl != TRUE)
         constructor(arg);
     *oncectrl = TRUE;
