@@ -1,6 +1,6 @@
 /*
 **  GNU Pth - The GNU Portable Threads
-**  Copyright (c) 1999-2002 Ralf S. Engelschall <rse@engelschall.com>
+**  Copyright (c) 1999-2003 Ralf S. Engelschall <rse@engelschall.com>
 **
 **  This file is part of GNU Pth, a non-preemptive thread scheduling
 **  library which can be found at http://www.gnu.org/software/pth/.
@@ -315,7 +315,7 @@ intern int pth_mctx_set(
         return FALSE;
     sigaltstack(NULL, &ss);
     if (!(ss.ss_flags & SS_DISABLE))
-        return_errno(FALSE, EIO);
+        return pth_error(FALSE, EIO);
     if (!(oss.ss_flags & SS_DISABLE))
         sigaltstack(&oss, NULL);
 #elif PTH_MCTX_STK(ss)
