@@ -1,6 +1,6 @@
 /*
 **  GNU Pth - The GNU Portable Threads
-**  Copyright (c) 1999-2002 Ralf S. Engelschall <rse@engelschall.com>
+**  Copyright (c) 1999-2003 Ralf S. Engelschall <rse@engelschall.com>
 **
 **  This file is part of GNU Pth, a non-preemptive thread scheduling
 **  library which can be found at http://www.gnu.org/software/pth/.
@@ -44,7 +44,7 @@
 /* initialize ring; O(1) */
 intern void pth_ring_init(pth_ring_t *r)
 {
-    if (r == NULL) 
+    if (r == NULL)
         return;
     r->r_hook  = NULL;
     r->r_nodes = 0;
@@ -228,7 +228,7 @@ intern int pth_ring_contains(pth_ring_t *r, pth_ringnode_t *rns)
     int rc;
 
     if (r == NULL || rns == NULL)
-        return_errno(FALSE, EINVAL);
+        return pth_error(FALSE, EINVAL);
     rc = FALSE;
     rn = r->r_hook;
     if (rn != NULL) {
