@@ -1,6 +1,6 @@
 /*
 **  GNU Pth - The GNU Portable Threads
-**  Copyright (c) 1999-2002 Ralf S. Engelschall <rse@engelschall.com>
+**  Copyright (c) 1999-2003 Ralf S. Engelschall <rse@engelschall.com>
 **
 **  This file is part of GNU Pth, a non-preemptive thread scheduling
 **  library which can be found at http://www.gnu.org/software/pth/.
@@ -47,16 +47,15 @@ static pth_time_t   pth_loadticknext;
 static pth_time_t   pth_loadtickgap = PTH_TIME(1,0);
 
 /* initialize the scheduler ingredients */
-intern void pth_scheduler_init(void)
+intern int pth_scheduler_init(void)
 {
     /* create the internal signal pipe */
-    if (pipe(pth_sigpipe) == -1) {
-        fprintf(stderr, "**Pth** INIT: Cannot create internal pipe: %s\n",
-                strerror(errno));
-        abort();
-    }
-    pth_fdmode(pth_sigpipe[0], PTH_FDMODE_NONBLOCK);
-    pth_fdmode(pth_sigpipe[1], PTH_FDMODE_NONBLOCK);
+    if (pipe(pth_sigpipe) == -1)
+        return pth_error(FALSE, errno);
+    if (pth_fdmode(pth_sigpipe[0], PTH_FDMODE_NONBLOCK) == PTH_FDMODE_ERROR)
+        return pth_error(FALSE, errno);
+    if (pth_fdmode(pth_sigpipe[1], PTH_FDMODE_NONBLOCK) == PTH_FDMODE_ERROR)
+        return pth_error(FALSE, errno);
 
     /* initialize the essential threads */
     pth_sched   = NULL;
@@ -72,7 +71,8 @@ intern void pth_scheduler_init(void)
     /* initialize load support */
     pth_loadval = 1.0;
     pth_time_set(&pth_loadticknext, PTH_TIME_NOW);
-    return;
+
+    return TRUE;
 }
 
 /* drop all threads (except for the currently active one) */
@@ -81,27 +81,27 @@ intern void pth_scheduler_drop(void)
     pth_t t;
 
     /* clear the new queue */
-    while ((t = pth_pqueue_delmax(&pth_NQ)) != NULL);
+    while ((t = pth_pqueue_delmax(&pth_NQ)) != NULL)
         pth_tcb_free(t);
     pth_pqueue_init(&pth_NQ);
 
     /* clear the ready queue */
-    while ((t = pth_pqueue_delmax(&pth_RQ)) != NULL);
+    while ((t = pth_pqueue_delmax(&pth_RQ)) != NULL)
         pth_tcb_free(t);
     pth_pqueue_init(&pth_RQ);
 
     /* clear the waiting queue */
-    while ((t = pth_pqueue_delmax(&pth_WQ)) != NULL);
+    while ((t = pth_pqueue_delmax(&pth_WQ)) != NULL)
         pth_tcb_free(t);
     pth_pqueue_init(&pth_WQ);
 
     /* clear the suspend queue */
-    while ((t = pth_pqueue_delmax(&pth_SQ)) != NULL);
+    while ((t = pth_pqueue_delmax(&pth_SQ)) != NULL)
         pth_tcb_free(t);
     pth_pqueue_init(&pth_SQ);
 
     /* clear the dead queue */
-    while ((t = pth_pqueue_delmax(&pth_DQ)) != NULL);
+    while ((t = pth_pqueue_delmax(&pth_DQ)) != NULL)
         pth_tcb_free(t);
     pth_pqueue_init(&pth_DQ);
     return;
@@ -123,9 +123,9 @@ intern void pth_scheduler_kill(void)
  * Update the average scheduler load.
  *
  * This is called on every context switch, but we have to adjust the
- * average load value every second, only. When we're called more than
+ * average load value every second, only. If we're called more than
  * once per second we handle this by just calculating anything once
- * and then do NOPs until the next ticks is over. When the scheduler
+ * and then do NOPs until the next ticks is over. If the scheduler
  * waited for more than once second (or a thread CPU burst lasted for
  * more than once second) we simulate the missing calculations. That's
  * no problem because we can assume that the number of ready threads
@@ -240,6 +240,7 @@ intern void *pth_scheduler(void *dummy)
         pth_time_add(&pth_sched->running, &running);
 
         /* ** ENTERING THREAD ** - by switching the machine context */
+        pth_current->dispatches++;
         pth_mctx_switch(&pth_sched->mctx, &pth_current->mctx);
 
         /* update scheduler times */
@@ -319,7 +320,7 @@ intern void *pth_scheduler(void *dummy)
         }
 
         /*
-         * When previous thread is now marked as dead, kick it out
+         * If previous thread is now marked as dead, kick it out
          */
         if (pth_current->state == PTH_STATE_DEAD) {
             pth_debug2("pth_scheduler: marking thread \"%s\" as dead", pth_current->name);
@@ -331,7 +332,7 @@ intern void *pth_scheduler(void *dummy)
         }
 
         /*
-         * When thread wants to wait for an event
+         * If thread wants to wait for an event
          * move it to waiting queue now
          */
         if (pth_current != NULL && pth_current->state == PTH_STATE_WAITING) {
@@ -367,7 +368,7 @@ intern void *pth_scheduler(void *dummy)
 }
 
 /*
- * Look whether some events already occurred and move
+ * Look whether some events already occurred (or failed) and move
  * corresponding threads from waiting queue back to ready queue.
  */
 intern void pth_sched_eventmanager(pth_time_t *now, int dopoll)
@@ -440,7 +441,7 @@ intern void pth_sched_eventmanager(pth_time_t *now, int dopoll)
         /* ...check whether events occurred */
         ev = evh = t->events;
         do {
-            if (!ev->ev_occurred) {
+            if (ev->ev_status == PTH_STATUS_PENDING) {
                 this_occurred = FALSE;
 
                 /* Filedescriptor I/O */
@@ -558,7 +559,7 @@ intern void pth_sched_eventmanager(pth_time_t *now, int dopoll)
                 /* tag event if it has occurred */
                 if (this_occurred) {
                     pth_debug2("pth_sched_eventmanager: [non-I/O] event occurred for thread \"%s\"", t->name);
-                    ev->ev_occurred = TRUE;
+                    ev->ev_status = PTH_STATUS_OCCURRED;
                     any_occurred = TRUE;
                 }
             }
@@ -632,7 +633,7 @@ intern void pth_sched_eventmanager(pth_time_t *now, int dopoll)
             /* it was an explicit timer event, standing for its own */
             pth_debug2("pth_sched_eventmanager: [timeout] event occurred for thread \"%s\"",
                        nexttimer_thread->name);
-            nexttimer_ev->ev_occurred = TRUE;
+            nexttimer_ev->ev_status = PTH_STATUS_OCCURRED;
         }
     }
 
@@ -650,7 +651,7 @@ intern void pth_sched_eventmanager(pth_time_t *now, int dopoll)
     }
 
     /* now comes the final cleanup loop where we've to
-       do two jobs: first we've to the late handling of the fd I/O events and
+       do two jobs: first we've to do the late handling of the fd I/O events and
        additionally if a thread has one occurred event, we move it from the
        waiting queue to the ready queue */
 
@@ -667,7 +668,7 @@ intern void pth_sched_eventmanager(pth_time_t *now, int dopoll)
                 /*
                  * Late handling for still not occured events
                  */
-                if (!ev->ev_occurred) {
+                if (ev->ev_status == PTH_STATUS_PENDING) {
                     /* Filedescriptor I/O */
                     if (ev->ev_type == PTH_EVENT_FD) {
                         if (   (   ev->ev_goal & PTH_UNTIL_FD_READABLE
@@ -678,7 +679,34 @@ intern void pth_sched_eventmanager(pth_time_t *now, int dopoll)
                                 && FD_ISSET(ev->ev_args.FD.fd, &efds)) ) {
                             pth_debug2("pth_sched_eventmanager: "
                                        "[I/O] event occurred for thread \"%s\"", t->name);
-                            ev->ev_occurred = TRUE;
+                            ev->ev_status = PTH_STATUS_OCCURRED;
+                        }
+                        else if (rc < 0) {
+                            /* re-check particular filedescriptor */
+                            int rc2;
+                            if (ev->ev_goal & PTH_UNTIL_FD_READABLE)
+                                FD_SET(ev->ev_args.FD.fd, &rfds);
+                            if (ev->ev_goal & PTH_UNTIL_FD_WRITEABLE)
+                                FD_SET(ev->ev_args.FD.fd, &wfds);
+                            if (ev->ev_goal & PTH_UNTIL_FD_EXCEPTION)
+                                FD_SET(ev->ev_args.FD.fd, &efds);
+                            pth_time_set(&delay, PTH_TIME_ZERO);
+                            while ((rc2 = pth_sc(select)(ev->ev_args.FD.fd+1, &rfds, &wfds, &efds, &delay)) < 0
+                                   && errno == EINTR) ;
+                            if (rc2 > 0) {
+                                /* cleanup afterwards for next iteration */
+                                FD_CLR(ev->ev_args.FD.fd, &rfds);
+                                FD_CLR(ev->ev_args.FD.fd, &wfds);
+                                FD_CLR(ev->ev_args.FD.fd, &efds);
+                            } else if (rc2 < 0) {
+                                /* cleanup afterwards for next iteration */
+                                FD_ZERO(&rfds);
+                                FD_ZERO(&wfds);
+                                FD_ZERO(&efds);
+                                ev->ev_status = PTH_STATUS_FAILED;
+                                pth_debug2("pth_sched_eventmanager: "
+                                           "[I/O] event failed for thread \"%s\"", t->name);
+                            }
                         }
                     }
                     /* Filedescriptor Set I/O */
@@ -693,10 +721,40 @@ intern void pth_sched_eventmanager(pth_time_t *now, int dopoll)
                                                     ev->ev_args.SELECT.efds, &efds);
                             if (ev->ev_args.SELECT.n != NULL)
                                 *(ev->ev_args.SELECT.n) = n;
-                            ev->ev_occurred = TRUE;
+                            ev->ev_status = PTH_STATUS_OCCURRED;
                             pth_debug2("pth_sched_eventmanager: "
                                        "[I/O] event occurred for thread \"%s\"", t->name);
                         }
+                        else if (rc < 0) {
+                            /* re-check particular filedescriptor set */
+                            int rc2;
+                            fd_set *prfds = NULL;
+                            fd_set *pwfds = NULL;
+                            fd_set *pefds = NULL;
+                            fd_set trfds;
+                            fd_set twfds;
+                            fd_set tefds;
+                            if (ev->ev_args.SELECT.rfds) {
+                                memcpy(&trfds, ev->ev_args.SELECT.rfds, sizeof(rfds));
+                                prfds = &trfds;
+                            }
+                            if (ev->ev_args.SELECT.wfds) {
+                                memcpy(&twfds, ev->ev_args.SELECT.wfds, sizeof(wfds));
+                                pwfds = &twfds;
+                            }
+                            if (ev->ev_args.SELECT.efds) {
+                                memcpy(&tefds, ev->ev_args.SELECT.efds, sizeof(efds));
+                                pefds = &tefds;
+                            }
+                            pth_time_set(&delay, PTH_TIME_ZERO);
+                            while ((rc2 = pth_sc(select)(ev->ev_args.SELECT.nfd+1, prfds, pwfds, pefds, &delay)) < 0
+                                   && errno == EINTR) ;
+                            if (rc2 < 0) {
+                                ev->ev_status = PTH_STATUS_FAILED;
+                                pth_debug2("pth_sched_eventmanager: "
+                                           "[I/O] event failed for thread \"%s\"", t->name);
+                            }
+                        }
                     }
                     /* Signal Set */
                     else if (ev->ev_type == PTH_EVENT_SIGS) {
@@ -708,7 +766,7 @@ intern void pth_sched_eventmanager(pth_time_t *now, int dopoll)
                                     pth_debug2("pth_sched_eventmanager: "
                                                "[signal] event occurred for thread \"%s\"", t->name);
                                     sigdelset(&pth_sigraised, sig);
-                                    ev->ev_occurred = TRUE;
+                                    ev->ev_status = PTH_STATUS_OCCURRED;
                                 }
                             }
                         }
@@ -730,7 +788,7 @@ intern void pth_sched_eventmanager(pth_time_t *now, int dopoll)
                 }
 
                 /* local to global mapping */
-                if (ev->ev_occurred)
+                if (ev->ev_status != PTH_STATUS_PENDING)
                     any_occurred = TRUE;
             } while ((ev = ev->ev_next) != evh);
         }
