@@ -1,6 +1,6 @@
 /*
 **  GNU Pth - The GNU Portable Threads
-**  Copyright (c) 1999-2002 Ralf S. Engelschall <rse@engelschall.com>
+**  Copyright (c) 1999-2003 Ralf S. Engelschall <rse@engelschall.com>
 **
 **  This file is part of GNU Pth, a non-preemptive thread scheduling
 **  library which can be found at http://www.gnu.org/software/pth/.
@@ -92,13 +92,14 @@ intern char *pth_util_cpystrn(char *dst, const char *src, size_t dst_size)
 }
 
 /* check whether a file-descriptor is valid */
-#if cpp
-#if !defined(FD_SETSIZE)
-#define FD_SETSIZE 1024
-#endif
-#define pth_util_fd_valid(fd) \
-    ((fd) >= 0 && (fd) <= (FD_SETSIZE-1))
-#endif
+intern int pth_util_fd_valid(int fd)
+{
+    if (fd < 0 || fd >= FD_SETSIZE)
+        return FALSE;
+    if (fcntl(fd, F_GETFL) == -1 && errno == EBADF)
+        return FALSE;
+    return TRUE;
+}
 
 /* merge input fd set into output fds */
 intern void pth_util_fds_merge(int nfd,
