@@ -1,6 +1,6 @@
 /*
 **  GNU Pth - The GNU Portable Threads
-**  Copyright (c) 1999-2002 Ralf S. Engelschall <rse@engelschall.com>
+**  Copyright (c) 1999-2003 Ralf S. Engelschall <rse@engelschall.com>
 **
 **  This file is part of GNU Pth, a non-preemptive thread scheduling
 **  library which can be found at http://www.gnu.org/software/pth/.
@@ -22,7 +22,9 @@
 **
 **  test_pthread.c: Pth test program (pthread API)
 */
-
+                             /* ``You can check out any time you
+                                like, but you can never leave.''
+                                -- The Eagles, Hotel California */
 #ifdef GLOBAL
 #include <pthread.h>
 #else
