@@ -1,6 +1,6 @@
 /*
 **  GNU Pth - The GNU Portable Threads
-**  Copyright (c) 1999-2002 Ralf S. Engelschall <rse@engelschall.com>
+**  Copyright (c) 1999-2003 Ralf S. Engelschall <rse@engelschall.com>
 **
 **  This file is part of GNU Pth, a non-preemptive thread scheduling
 **  library which can be found at http://www.gnu.org/software/pth/.
@@ -22,7 +22,10 @@
 **
 **  test_std.c: Pth standard test program
 */
-
+                             /* ``Understanding a problem is knowing why
+                                it is hard to solve it, and why the most
+                                straightforward approaches won't work.''
+                                                  -- Karl Popper        */
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
